#!/usr/bin/env python
"""
Factorizacin LU de una matriz
                          By: KnX
"""


import Matrix
from numpy.numarray import dot

def imprime_Matriz(a):
    """Imprime de forma elegante una matriz."""
    n, m = a.shape
    for i in range(n):
        for j in range(m):
            print '%2.3f' % a[i,j],
        print

def imprime_Vector(a):
    """Imprime de forma elegante un vector."""

    for i in range(len(a)):
        print '%2.3f' % a[i],
        print


def permuta_Matriz(perm, n):
    """
    Crea una matriz de permutaciones a partir de la permutacion perm.
    perm tiene el formato [(i0,j0),(i1,j1),...].
    """
    P = Matrix.identity(n)

    for i,j in perm:
        intercambia_filas(P, i, j)

    return P


def desempaqueta(A):
    """
    Devuelve las matrices L,U a partir de la forma empaquetada en que
    las calcula GSL_LU.
    """
    n, m = A.shape

    # crea las matrices L y U inicializadas a cero
    L = Matrix.Matrix([[0.0]*n]*n)
    U = Matrix.Matrix([[0.0]*n]*n)

    #xrange es como range pero en lugar de generar una lista genera
    #numeros en el rango solicitado. Es algo mas eficiente que
    #range cuando se trata de generar bucles
    
    for i in xrange(n):   
        L[i,i] = 1.               # pone los elementos diagonales de L a 1


    for i in xrange(n):
        for j in xrange(n):
            if i > j:
                L[i,j] = A[i,j] #copia en L la parte diagonal inferior de A    
            else:
                U[i,j] = A[i,j] #copia en U la parte diagonal superior de A    

    return L, U   


def intercambia_filas(A, i, j):
    """
    Intercambia las filas i <--> j de la matriz A.
    """
    nfil, ncol = A.shape

    for k in xrange(ncol):
        A[i,k], A[j,k] = A[j,k], A[i,k]


def GSL_LU(A):
    """
    Factoriza una matriz cuadrada A en

    P A = L U
   
    """

    # La matriz debe ser cuadrada
    n, m = A.shape
    assert n == m

    perm = []
    signum = 1

    for j in xrange(n-1):
        # Encuentra el maximo de la j-esima columna
        maximo = abs(A[j,j])
        pivote = j

        for i in xrange(j+1, n):
            aij = abs(A[i,j])
            if aij > maximo:
                maximo = aij
                pivote = i

        # Intercambia filas si es necesario, para usar el mayor pivote
        if pivote != j:
            intercambia_filas(A, j, pivote)
            perm.append((j,pivote))
            signum = -signum

        # Calcula y empaqueta
        ajj = A[j,j]   # esquina superior izquierda
        if (ajj != 0.0):
            for i in xrange(j+1, n):
                aij = A[i,j] / ajj   # divide los elementos de la columna por
                A[i,j] = aij         # el pivote

                for k in xrange(j+1, n):  # halla los elementos de U a partir
                    aik = A[i, k]         # de la fila
                    ajk = A[j, k]
                    A[i,k] = aik - aij * ajk

    return perm, signum  # A tambien ha cambiado




def LU(A):
    """Factorizacin LU de una matriz."""

    n, m = A.shape
    assert n == m

    L = Matrix.Matrix([[0.0]*n]*n)
    U = Matrix.Matrix([[0.0]*n]*n)
    # En realidad podriamos empaquetar toda la informacion en una sola matriz

    for i in range(n):
        L[i,i] = 1.
    
    for i in range(n):  # Para cada "marco" superior izquierdo
        for j in range(i, n):    # Rellena la U a partir de la fila
            suma = 0.0
            for k in range(0, j):
                suma += L[i,k] * U[k,j]
            U[i,j] = A[i,j] - suma

        for j in range(i+1, n):  # Rellena la L a partir de la columna
            suma = 0.0
            for k in range(0, i):
                suma += L[j,k]*U[k,i]
            L[j,i] = (A[j,i] - suma) / U[i,i]

    return L, U

def prostituciones(a,b):
  n=len(a)
  for k in range(1,n):
    b[k]=b[k] - dot(a[k,0:k],b[0:k])
  
  #for k in range(n-1,-1,-1):
   # b[k]=(b[k]-dot(a[k,k+1],b[k+1:n]))/a[k,k]
  return b


if __name__ == '__main__':    
    n = 3  # usaremos matrices 3x3

    # Matriz de coeficientes. 
    A = Matrix.Matrix([[0.0]*n]*n)  # los nueve valores iniciales son cero

    # ACA HAY Q COMENTAR ESTOS 2 FOR Y REEMPLAZARLA POR LA LINEA Q ESTA COMENTADA ABAJO , OSE LA Q TIENE LA A= ....

    #for i in range(n):
     #   for j in range(n):
      #      A[i,j] = 1 + i*n + j

    A[0,0] =1
    A[0,1] =4
    A[0,2] =1
    A[1,0] =1
    A[1,1] =6
    A[1,2] =-1
    A[2,0] =2
    A[2,1] =-1
    A[2,2] =2
    
    b=[7,13,5]
 
    # Descomposicion LU
    L, U = LU(A)

    print 'Con el primer algoritmo:'
    print 'A ='
    imprime_Matriz(A)
    print
    print 'L, U ='
    for m in [L, U]:
        imprime_Matriz(m)
    print 'A - L * U'
    imprime_Matriz(A - L * U)
     
    print "SOLUCION"
    imprimi=prostituciones(L,b)
    imprimi
    